///////////////////////////////////////////////////////////////////////////////
// File    : placer.h
// Content : treemap placer
// History : 10-Apr-2006 - [rb] creation
//           19-Oct-2007 - [rb] added CeCIIL licence, prepared distro
///////////////////////////////////////////////////////////////////////////////

/* licence ********************************************************************
Copyright Renaud Blanch [http://iihm.imag.fr/blanch/]

This software is a computer program whose purpose is to explore new interaction
techniques for zoomable treemaps.

This software is governed by the CeCILL license under French law and abiding by 
the rules of distribution of free software. You can  use, modify and/ or 
redistribute the software under the terms of the CeCILL license as circulated  
at the following URL [http://www.cecill.info]. 

As a counterpart to the access to the source code and  rights to copy, modify 
and redistribute granted by the license, users are provided only with a limited 
warranty  and the software's author,  the holder of the economic rights,  and 
the successive licensors  have only limited liability. 

The fact that you are presently reading this means that you have had knowledge 
of the CeCILL license and that you accept its terms.
******************************************************************************/

#if !defined(PLACER_H)
#define PLACER_H


/* includes ******************************************************************/

#include "rectangle.h"

#include <vector>

typedef std::vector< double >    WeightList;
typedef std::vector< Rectangle > RectangleList;


/* placers declaration *******************************************************/

/*
	A placer takes a rectangle and splits it into non-overlapping 
	subrectangles having their size proportionnal to the weights given 
	as input.
*/

struct Placer {
	mutable int count;
	Placer() : count(0) {}
	
	static Placer *create(char type);
	
	virtual ~Placer() {}
	virtual RectangleList split(const Rectangle &box, int level, double weight,
	                            const WeightList &weights) const = 0;
};

struct SliceAndDice : Placer {
	virtual RectangleList split(const Rectangle &box, int level, double weight,
	                            const WeightList &weights) const;
};

struct Squarified : Placer {
	virtual RectangleList split(const Rectangle &box, int level, double weight,
	                            const WeightList &weights) const;
};


#endif // !defined(PLACER_H)

