///////////////////////////////////////////////////////////////////////////////
// File    : view.cpp
// Content : view abstraction
// History : 10-Apr-2006 - [rb] creation
//           19-Oct-2007 - [rb] added CeCIIL licence, prepared distro
///////////////////////////////////////////////////////////////////////////////

/* licence ********************************************************************
Copyright Renaud Blanch [http://iihm.imag.fr/blanch/]

This software is a computer program whose purpose is to explore new interaction
techniques for zoomable treemaps.

This software is governed by the CeCILL license under French law and abiding by 
the rules of distribution of free software. You can  use, modify and/ or 
redistribute the software under the terms of the CeCILL license as circulated  
at the following URL [http://www.cecill.info]. 

As a counterpart to the access to the source code and  rights to copy, modify 
and redistribute granted by the license, users are provided only with a limited 
warranty  and the software's author,  the holder of the economic rights,  and 
the successive licensors  have only limited liability. 

The fact that you are presently reading this means that you have had knowledge 
of the CeCILL license and that you accept its terms.
******************************************************************************/

/* includes ******************************************************************/

#include "view.h"

#include <iostream>
#include <cassert>
#include <cmath>
using namespace std;

#if defined(__APPLE__)
#	include <GLUT/glut.h>
#else
#	include <GL/glut.h>
#endif


/* static data ***************************************************************/

View::Animation View::animation;
int View::step = 15;


/* view declaration **********************************************************/

void View::setGLMatrix() {
	crop(Rectangle().dilate(double(MARGIN)/W(), double(MARGIN)/H()));
	glScalef(1./w(), 1./h(), 1);
	glTranslatef(-x0, -y0, -target().depth());
}

void View::pan(double dx, double dy) {
	x0 += dx; x1 += dx;
	y0 += dy; y1 += dy;
}

void View::zoom(double dsx, double dsy, double x, double y) {
	x0 = x + (x0 - x)/dsx; x1 = x + (x1 - x)/dsx;
	y0 = y + (y0 - y)/dsy; y1 = y + (y1 - y)/dsy; 
}

void View::zoom(double dsx, double dsy) {
	zoom(dsx, dsy, (x0 + x1)/2, (y0 + y1)/2);
}


void View::zoom(const Rectangle &r) {
	animation.clear();
	
	Rectangle target(r);
	target.dilate(MARGIN*target.w()/W(), MARGIN*target.h()/H());

	step = int(abs(depth() - target.depth()) * STEPS_PER_LEVEL + 1);
	step = min(step, step_max);
	step = max(step, step_min);
	
	double dsx = pow(target.w()/w(), (double)(1.)/step);
	double dsy = pow(target.h()/h(), (double)(1.)/step);
	assert(dsx > 0. and dsy > 0.);
	
	double dx0 = target.x0 - x0;
	if(dsx != 1.) dx0 *= (1-dsx)/(1-pow(dsx, step));
	else          dx0 /= step;
	
	double dy0 = target.y0 - y0;
	if(dsy != 1.) dy0 *= (1-dsy)/(1-pow(dsy, step));
	else          dy0 /= step;
	
	animation.push_back(target);	
	for(int i = step - 1; i >= 0; --i) {
		target.x0 -= dx0 * pow(dsx, i);
		target.y0 -= dy0 * pow(dsy, i);
		target.x1 = target.x0 + w() * pow(dsx, i);
		target.y1 = target.y0 + h() * pow(dsy, i);
		animation.push_back(target);	
	}
	
	glutTimerFunc(1, autozoom, reinterpret_cast< int >(this));	
}

void View::autozoom(int v) {
	if(animation.empty()) return;

	Rectangle &view = *reinterpret_cast< View * >(v);
	view = animation.back();
	animation.pop_back();
	
	glutPostRedisplay();

	glutTimerFunc(20, autozoom, v);	
}
